package bookExamples.ch26Graphics.draw2d.shapes;


import bookExamples.ch26Graphics.draw2d.*;
import gui.In;
import gui.run.RunButton;
import gui.run.RunSlider;

import javax.swing.*;
import java.awt.*;
import java.awt.geom.AffineTransform;
import java.awt.geom.Ellipse2D;
import java.beans.PropertyEditor;

public class Circle2d extends DJShape
        implements PropertyEditor, Intersects, Movable, Containment {
    private int diameter = 1;
    private Vec2d venterV2d = new Vec2d(0, 0);
    private double radiusSq;

    public Circle2d(int x1, int y1, int diameter) {
        super(x1, y1, x1 + diameter, y1 + diameter);
        this.setX1(x1);
        this.setY1(y1);
        this.setDiameter(diameter);
        int radius = this.getDiameter() / 2;
        setRadiusSq(radius * radius);
        setCenter();

    }

    public Component getCustomEditor() {
        JPanel jp = new JPanel();
        jp.setLayout(new FlowLayout());
        jp.add(new JLabel("diameter"));
        jp.add(new RunSlider(RunSlider.HORIZONTAL, 10, 2000, getDiameter()) {
            public void run() {
                setDiameter(getValue());
                getPcs().firePropertyChange("Circle2d", Circle2d.this, null);
            }
        });
        jp.add(new RunButton("setForeground") {
            public void run() {
                Circle2d.this.setForeground(In.getColor());

                getPcs().firePropertyChange("size", Circle2d.this, null);
            }
        });
        jp.add(getRunAffinePanel());

        jp.setLayout(new FlowLayout());

        return jp;
    }


    public DJShape getInstance(int x1, int y1, int x2, int y2) {
        Circle2d circle2d = new Circle2d(x1, y1, x2, y2);
        setForeground(Color.MAGENTA);
        return circle2d;
    }


    public Circle2d(int _x1, int _y1, int _x2, int _y2) {
        super(_x1, _y1, _x2, _y2);
        setX1(_x1);
        setY1(_y1);
        int dx = getX1() - _x2;
        int dy = getY1() - _y2;
        setDiameter((int) Math.sqrt(dx * dx + dy * dy));
        if (_x1 > _x2) setX1(_x2);
        if (_y1 > _y2) setY1(_y2);
        int radius = getDiameter() / 2;
        setRadiusSq(radius * radius);
        setCenter();
    }

    private void setCenter() {
        int radius = getDiameter() / 2;
        getVenterV2d().v[0] = getX1() + radius;
        getVenterV2d().v[1] = getY1() + radius;
    }

    public Point getCenter() {
        return new Point((int) getVenterV2d().v[0], (int) getVenterV2d().v[1]);
    }

    public void draw(Graphics g) {
        //saveState(g);

        Point pc = getCenter();

        Graphics2D g2d = (Graphics2D) g;

        AffineTransform old = g2d.getTransform();
        AffineTransform affineTransform = getAffineTransform();
        g2d.transform(affineTransform);
        g2d.setColor(getForeground());
        int radius = getDiameter() / 2;
        setX1(pc.x - radius);
        setY1(pc.y - radius);
        g2d.drawOval(getX1(), getY1(), getDiameter(), getDiameter());

        g2d.fillOval(pc.x, pc.y, 2, 2);
        g2d.drawString("(" + pc.x + "," + pc.y + ")", pc.x + 3, pc.y + 3);
        g2d.drawLine(pc.x, pc.y, pc.x - radius, pc.y);
        g2d.setTransform(old);

    }

    public String toString() {
        Point pc = getCenter();
        return
                "xc,yc,r=" + pc.x + "," + pc.y + "," + getDiameter();
    }


    public boolean inside(Vec2d p) {

        Vec2d n = new Vec2d(p);
        n.sub(getVenterV2d());
        if (n.dot(n) > getRadiusSq())
            return false;
        else
            return true;
    }

    public Vec2d intersect(Ray2d ray) {

        double t, /* where the ray intersects */
                // vDotV = distance square from sphere venterV2d to
                // ray origin
                vDotV,
                dDotV,
                thc;        /* length sqare of the half chord */

        // vector to the venterV2d of the sphere from
        // ray origin
        Vec2d v = new Vec2d(getVenterV2d().v[0], getVenterV2d().v[1]);
        boolean inside = false;
        //System.out.println(ray);

        /* use the closest approach algorithm */
        // v = v - ray.p
        //System.out.println(v);
        v.sub(ray.getOrigin());
        // loc = v^2
        vDotV = v.dot(v);
        // Are we inside the circle?
        if (vDotV <= getRadiusSq())
            inside = true;

        // dDotV = D dot V =
        // cos(angle between ray and vec to venterV2d);
        dDotV = v.dot(ray.getDirection());   /* find the closest approach */

        // object is behind the ray origin
        if ((!inside) && (dDotV <= 0.0))
            return null;

        /* compute the half chord square from the ray intersection to the
           intersection normal. */
        thc = (dDotV * dDotV) - vDotV + getRadiusSq();
        if (thc < 0.0)
            return null;   /* ray misses the sphere */

        /* find the ray intersection */
        if (inside)
            t = dDotV + Math.sqrt(thc);
        else
            t = dDotV - Math.sqrt(thc);

        return ray.vecOnLine(t);
    }

    public void move(double x, double y) {
        setVenterV2d(new Vec2d(x, y));
    }

    public boolean isPaintable() {
        return false;
    }

    public boolean supportsCustomEditor() {
        return true;
    }

    public Object getValue() {
        return null;
    }

    public void setValue(Object value) {
    }

    public String getAsText() {
        return null;
    }

    public String getJavaInitializationString() {
        return null;
    }

    public String[] getTags() {
        return new String[0];
    }

    public void setAsText(String text) throws IllegalArgumentException {
    }

    public void paintValue(Graphics gfx, Rectangle box) {
    }

    public int getDiameter() {
        return diameter;
    }

    public void setDiameter(int diameter) {
        this.diameter = diameter;
    }

    public double getRadiusSq() {
        return radiusSq;
    }

    public void setRadiusSq(double radiusSq) {
        this.radiusSq = radiusSq;
    }

    public Vec2d getVenterV2d() {
        return venterV2d;
    }

    public void setVenterV2d(Vec2d venterV2d) {
        this.venterV2d = venterV2d;
    }

    //todo this has a glich in it.
    public boolean contains(double x, double y) {
        Shape s = getShape();
        return s.contains(x,y);

    }

    public Shape getShape() {
        double d = getDiameter();
        Point pc = getCenter();
        Ellipse2D e2d = new Ellipse2D.Double(pc.x - d / 2, pc.y - d / 2, d, d);
        AffineTransform at = getAffineTransform();
        return at.createTransformedShape(e2d);
    }
}
